@@ -3,34 +3,22 @@
 #
 # This module is part of GitPython and is released under
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
-import tempfile
-import os
-import sys
-import subprocess
 import glob
 from io import BytesIO
-
+import os
 from stat import S_ISLNK
+import subprocess
+import tempfile
 
-from .typ import (
-    BaseIndexEntry,
-    IndexEntry,
+from git.compat import (
+    force_bytes,
+    defenc,
 )
-
-from .util import (
-    TemporaryFileSwap,
-    post_clear_cache,
-    default_index,
-    git_working_dir
-)
-
-import git.diff as diff
 from git.exc import (
     GitCommandError,
     CheckoutError,
     InvalidGitRepositoryError
 )
-
 from git.objects import (
     Blob,
     Submodule,
@@ -38,25 +26,21 @@ from git.objects import (
     Object,
     Commit,
 )
-
 from git.objects.util import Serializable
-from git.compat import (
-    izip,
-    xrange,
-    string_types,
-    force_bytes,
-    defenc,
-    mviter
-)
-
 from git.util import (
     LazyMixin,
     LockedFD,
     join_path_native,
     file_contents_ro,
     to_native_path_linux,
-    unbare_repo
+    unbare_repo,
+    to_bin_sha
 )
+from gitdb.base import IStream
+from gitdb.db import MemoryDB
+
+import git.diff as diff
+import os.path as osp
 
 from .fun import (
     entry_key,
@@ -68,10 +52,17 @@ from .fun import (
     S_IFGITLINK,
     run_commit_hook
 )
+from .typ import (
+    BaseIndexEntry,
+    IndexEntry,
+)
+from .util import (
+    TemporaryFileSwap,
+    post_clear_cache,
+    default_index,
+    git_working_dir
+)
 
-from gitdb.base import IStream
-from gitdb.db import MemoryDB
-from gitdb.util import to_bin_sha
 
 __all__ = ('IndexFile', 'CheckoutError')
 
@@ -118,28 +109,26 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             # read the current index
             # try memory map for speed
             lfd = LockedFD(self._file_path)
+            ok = False
             try:
                 fd = lfd.open(write=False, stream=False)
+                ok = True
             except OSError:
-                lfd.rollback()
                 # in new repositories, there may be no index, which means we are empty
-                self.entries = dict()
+                self.entries = {}
                 return
+            finally:
+                if not ok:
+                    lfd.rollback()
             # END exception handling
 
-            # Here it comes: on windows in python 2.5, memory maps aren't closed properly
-            # Hence we are in trouble if we try to delete a file that is memory mapped,
-            # which happens during read-tree.
-            # In this case, we will just read the memory in directly.
-            # Its insanely bad ... I am disappointed !
-            allow_mmap = (os.name != 'nt' or sys.version_info[1] > 5)
-            stream = file_contents_ro(fd, stream=True, allow_mmap=allow_mmap)
+            stream = file_contents_ro(fd, stream=True, allow_mmap=True)
 
             try:
                 self._deserialize(stream)
             finally:
                 lfd.rollback()
-                # The handles will be closed on desctruction
+                # The handles will be closed on destruction
             # END read from default index on demand
         else:
             super(IndexFile, self)._set_cache_(attr)
@@ -165,23 +154,24 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
 
     def _deserialize(self, stream):
         """Initialize this instance with index values read from the given stream"""
-        self.version, self.entries, self._extension_data, conten_sha = read_cache(stream)
+        self.version, self.entries, self._extension_data, _conten_sha = read_cache(stream)
         return self
 
     def _entries_sorted(self):
         """:return: list of entries, in a sorted fashion, first by path, then by stage"""
         return sorted(self.entries.values(), key=lambda e: (e.path, e.stage))
 
-    def _serialize(self, stream, ignore_tree_extension_data=False):
+    def _serialize(self, stream, ignore_extension_data=False):
         entries = self._entries_sorted()
-        write_cache(entries,
-                    stream,
-                    (ignore_tree_extension_data and None) or self._extension_data)
+        extension_data = self._extension_data
+        if ignore_extension_data:
+            extension_data = None
+        write_cache(entries, stream, extension_data)
         return self
 
     #} END serializable interface
 
-    def write(self, file_path=None, ignore_tree_extension_data=False):
+    def write(self, file_path=None, ignore_extension_data=False):
         """Write the current state to our file path or to the given one
 
         :param file_path:
@@ -190,9 +180,10 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             Please note that this will change the file_path of this index to
             the one you gave.
 
-        :param ignore_tree_extension_data:
+        :param ignore_extension_data:
             If True, the TREE type extension data read in the index will not
-            be written to disk. Use this if you have altered the index and
+            be written to disk. NOTE that no extension data is actually written.
+            Use this if you have altered the index and
             would like to use git-write-tree afterwards to create a tree
             representing your written changes.
             If this data is present in the written index, git-write-tree
@@ -208,7 +199,13 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
         lfd = LockedFD(file_path or self._file_path)
         stream = lfd.open(write=True, stream=True)
 
-        self._serialize(stream, ignore_tree_extension_data)
+        ok = False
+        try:
+            self._serialize(stream, ignore_extension_data)
+            ok = True
+        finally:
+            if not ok:
+                lfd.rollback()
 
         lfd.commit()
 
@@ -271,8 +268,8 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
 
         inst = cls(repo)
         # convert to entries dict
-        entries = dict(izip(((e.path, e.stage) for e in base_entries),
-                            (IndexEntry.from_base(e) for e in base_entries)))
+        entries = dict(zip(((e.path, e.stage) for e in base_entries),
+                           (IndexEntry.from_base(e) for e in base_entries)))
 
         inst.entries = entries
         return inst
@@ -315,7 +312,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
         if len(treeish) == 0 or len(treeish) > 3:
             raise ValueError("Please specify between 1 and 3 treeish, got %i" % len(treeish))
 
-        arg_list = list()
+        arg_list = []
         # ignore that working tree and index possibly are out of date
         if len(treeish) > 1:
             # drop unmerged entries when reading our index and merging
@@ -341,7 +338,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             index.entries       # force it to read the file as we will delete the temp-file
             del(index_handler)  # release as soon as possible
         finally:
-            if os.path.exists(tmp_index):
+            if osp.exists(tmp_index):
                 os.remove(tmp_index)
         # END index merge handling
 
@@ -361,21 +358,40 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
         rs = r + os.sep
         for path in paths:
             abs_path = path
-            if not os.path.isabs(abs_path):
-                abs_path = os.path.join(r, path)
+            if not osp.isabs(abs_path):
+                abs_path = osp.join(r, path)
             # END make absolute path
 
-            # resolve globs if possible
-            if '?' in path or '*' in path or '[' in path:
-                for f in self._iter_expand_paths(glob.glob(abs_path)):
-                    yield f.replace(rs, '')
-                continue
+            try:
+                st = os.lstat(abs_path)     # handles non-symlinks as well
+            except OSError:
+                # the lstat call may fail as the path may contain globs as well
+                pass
+            else:
+                if S_ISLNK(st.st_mode):
+                    yield abs_path.replace(rs, '')
+                    continue
+            # end check symlink
+
+            # if the path is not already pointing to an existing file, resolve globs if possible
+            if not os.path.exists(path) and ('?' in path or '*' in path or '[' in path):
+                resolved_paths = glob.glob(abs_path)
+                # not abs_path in resolved_paths:
+                #   a glob() resolving to the same path we are feeding it with
+                #   is a glob() that failed to resolve. If we continued calling
+                #   ourselves we'd endlessly recurse. If the condition below
+                #   evaluates to true then we are likely dealing with a file
+                #   whose name contains wildcard characters.
+                if abs_path not in resolved_paths:
+                    for f in self._iter_expand_paths(glob.glob(abs_path)):
+                        yield f.replace(rs, '')
+                    continue
             # END glob handling
             try:
-                for root, dirs, files in os.walk(abs_path, onerror=raise_exc):
+                for root, _dirs, files in os.walk(abs_path, onerror=raise_exc):
                     for rela_file in files:
                         # add relative paths only
-                        yield os.path.join(root.replace(rs, ''), rela_file)
+                        yield osp.join(root.replace(rs, ''), rela_file)
                     # END for each file in subdir
                 # END for each subdirectory
             except OSError:
@@ -405,7 +421,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
         try:
             proc.stdin.write(("%s\n" % filepath).encode(defenc))
         except IOError:
-            # pipe broke, usually because some error happend
+            # pipe broke, usually because some error happened
             raise fmakeexc()
         # END write exception handling
         proc.stdin.flush()
@@ -422,7 +438,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             Function(t) returning True if tuple(stage, Blob) should be yielded by the
             iterator. A default filter, the BlobFilter, allows you to yield blobs
             only if they match a given list of paths. """
-        for entry in mviter(self.entries):
+        for entry in self.entries.values():
             blob = entry.to_blob(self.repo)
             blob.size = entry.size
             output = (entry.stage, blob)
@@ -443,11 +459,11 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             are at stage 3 will not have a stage 3 entry.
         """
         is_unmerged_blob = lambda t: t[0] != 0
-        path_map = dict()
+        path_map = {}
         for stage, blob in self.iter_blobs(is_unmerged_blob):
-            path_map.setdefault(blob.path, list()).append((stage, blob))
+            path_map.setdefault(blob.path, []).append((stage, blob))
         # END for each unmerged blob
-        for l in mviter(path_map):
+        for l in path_map.values():
             l.sort()
         return path_map
 
@@ -537,22 +553,24 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
     def _to_relative_path(self, path):
         """:return: Version of path relative to our git directory or raise ValueError
         if it is not within our git direcotory"""
-        if not os.path.isabs(path):
+        if not osp.isabs(path):
             return path
         if self.repo.bare:
             raise InvalidGitRepositoryError("require non-bare repository")
-        relative_path = path.replace(self.repo.working_tree_dir + os.sep, "")
-        if relative_path == path:
+        if not path.startswith(self.repo.working_tree_dir):
             raise ValueError("Absolute path %r is not in git repository at %r" % (path, self.repo.working_tree_dir))
-        return relative_path
+        return os.path.relpath(path, self.repo.working_tree_dir)
 
     def _preprocess_add_items(self, items):
         """ Split the items into two lists of path strings and BaseEntries. """
-        paths = list()
-        entries = list()
+        paths = []
+        entries = []
+        # if it is a string put in list
+        if isinstance(items, str):
+            items = [items]
 
         for item in items:
-            if isinstance(item, string_types):
+            if isinstance(item, str):
                 paths.append(self._to_relative_path(item))
             elif isinstance(item, (Blob, Submodule)):
                 entries.append(BaseIndexEntry.from_blob(item))
@@ -561,37 +579,36 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             else:
                 raise TypeError("Invalid Type: %r" % item)
         # END for each item
-        return (paths, entries)
+        return paths, entries
 
     def _store_path(self, filepath, fprogress):
         """Store file at filepath in the database and return the base index entry
         Needs the git_working_dir decorator active ! This must be assured in the calling code"""
         st = os.lstat(filepath)     # handles non-symlinks as well
-        stream = None
         if S_ISLNK(st.st_mode):
             # in PY3, readlink is string, but we need bytes. In PY2, it's just OS encoded bytes, we assume UTF-8
-            stream = BytesIO(force_bytes(os.readlink(filepath), encoding='utf-8'))
+            open_stream = lambda: BytesIO(force_bytes(os.readlink(filepath), encoding=defenc))
         else:
-            stream = open(filepath, 'rb')
-        # END handle stream
-        fprogress(filepath, False, filepath)
-        istream = self.repo.odb.store(IStream(Blob.type, st.st_size, stream))
-        fprogress(filepath, True, filepath)
+            open_stream = lambda: open(filepath, 'rb')
+        with open_stream() as stream:
+            fprogress(filepath, False, filepath)
+            istream = self.repo.odb.store(IStream(Blob.type, st.st_size, stream))
+            fprogress(filepath, True, filepath)
         return BaseIndexEntry((stat_mode_to_index_mode(st.st_mode),
                                istream.binsha, 0, to_native_path_linux(filepath)))
 
     @unbare_repo
     @git_working_dir
     def _entries_for_paths(self, paths, path_rewriter, fprogress, entries):
-        entries_added = list()
+        entries_added = []
         if path_rewriter:
             for path in paths:
-                if os.path.isabs(path):
+                if osp.isabs(path):
                     abspath = path
                     gitrelative_path = path[len(self.repo.working_tree_dir) + 1:]
                 else:
                     gitrelative_path = path
-                    abspath = os.path.join(self.repo.working_tree_dir, gitrelative_path)
+                    abspath = osp.join(self.repo.working_tree_dir, gitrelative_path)
                 # end obtain relative and absolute paths
 
                 blob = Blob(self.repo, Blob.NULL_BIN_SHA,
@@ -612,7 +629,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
         return entries_added
 
     def add(self, items, force=True, fprogress=lambda *args: None, path_rewriter=None,
-            write=True):
+            write=True, write_extension_data=False):
         """Add files from the working tree, specific blobs or BaseIndexEntries
         to the index.
 
@@ -625,6 +642,10 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
                 strings denote a relative or absolute path into the repository pointing to
                 an existing file, i.e. CHANGES, lib/myfile.ext, '/home/gitrepo/lib/myfile.ext'.
 
+                Absolute paths must start with working tree directory of this index's repository
+                to be considered valid. For example, if it was initialized with a non-normalized path, like
+                `/root/repo/../repo`, absolute paths to be added must start with `/root/repo/../repo`.
+
                 Paths provided like this must exist. When added, they will be written
                 into the object database.
 
@@ -685,8 +706,19 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             Please note that entry.path is relative to the git repository.
 
         :param write:
-                If True, the index will be written once it was altered. Otherwise
-                the changes only exist in memory and are not available to git commands.
+            If True, the index will be written once it was altered. Otherwise
+            the changes only exist in memory and are not available to git commands.
+
+        :param write_extension_data:
+            If True, extension data will be written back to the index. This can lead to issues in case
+            it is containing the 'TREE' extension, which will cause the `git commit` command to write an
+            old tree, instead of a new one representing the now changed index.
+            This doesn't matter if you use `IndexFile.commit()`, which ignores the `TREE` extension altogether.
+            You should set it to True if you intend to use `IndexFile.commit()` exclusively while maintaining
+            support for third-party extensions. Besides that, you can usually safely ignore the built-in
+            extensions when using GitPython on repositories that are not handled manually at all.
+            All current built-in extensions are listed here:
+            http://opensource.apple.com/source/Git/Git-26/src/git-htmldocs/technical/index-format.txt
 
         :return:
             List(BaseIndexEntries) representing the entries just actually added.
@@ -700,7 +732,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
         # automatically
         # paths can be git-added, for everything else we use git-update-index
         paths, entries = self._preprocess_add_items(items)
-        entries_added = list()
+        entries_added = []
         # This code needs a working tree, therefore we try not to run it unless required.
         # That way, we are OK on a bare repository as well.
         # If there are no paths, the rewriter has nothing to do either
@@ -759,7 +791,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             self.entries[(entry.path, 0)] = IndexEntry.from_base(entry)
 
         if write:
-            self.write()
+            self.write(ignore_extension_data=not write_extension_data)
         # END handle write
 
         return entries_added
@@ -767,11 +799,15 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
     def _items_to_rela_paths(self, items):
         """Returns a list of repo-relative paths from the given items which
         may be absolute or relative paths, entries or blobs"""
-        paths = list()
+        paths = []
+        # if string put in list
+        if isinstance(items, str):
+            items = [items]
+
         for item in items:
             if isinstance(item, (BaseIndexEntry, (Blob, Submodule))):
                 paths.append(self._to_relative_path(item.path))
-            elif isinstance(item, string_types):
+            elif isinstance(item, str):
                 paths.append(self._to_relative_path(item))
             else:
                 raise TypeError("Invalid item type: %r" % item)
@@ -804,19 +840,19 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
 
         :param working_tree:
             If True, the entry will also be removed from the working tree, physically
-            removing the respective file. This may fail if there are uncommited changes
+            removing the respective file. This may fail if there are uncommitted changes
             in it.
 
         :param kwargs:
             Additional keyword arguments to be passed to git-rm, such
-            as 'r' to allow recurive removal of
+            as 'r' to allow recursive removal of
 
         :return:
             List(path_string, ...) list of repository relative paths that have
             been removed effectively.
             This is interesting to know in case you have provided a directory or
             globs. Paths are relative to the repository. """
-        args = list()
+        args = []
         if not working_tree:
             args.append("--cached")
         args.append("--")
@@ -834,7 +870,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
     def move(self, items, skip_errors=False, **kwargs):
         """Rename/move the items, whereas the last item is considered the destination of
         the move operation. If the destination is a file, the first item ( of two )
-        must be a file as well. If the destination is a directory, it may be preceeded
+        must be a file as well. If the destination is a directory, it may be preceded
         by one or more directories or files.
 
         The working tree will be affected in non-bare repositories.
@@ -844,7 +880,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             for reference.
         :param skip_errors:
             If True, errors such as ones resulting from missing source files will
-            be skpped.
+            be skipped.
         :param kwargs:
             Additional arguments you would like to pass to git-mv, such as dry_run
             or force.
@@ -853,9 +889,9 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             A list of pairs, containing the source file moved as well as its
             actual destination. Relative to the repository root.
 
-        :raise ValueErorr: If only one item was given
+        :raise ValueError: If only one item was given
             GitCommandError: If git could not handle your request"""
-        args = list()
+        args = []
         if skip_errors:
             args.append('-k')
 
@@ -868,12 +904,12 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
 
         # first execute rename in dryrun so the command tells us what it actually does
         # ( for later output )
-        out = list()
+        out = []
         mvlines = self.repo.git.mv(args, paths, **kwargs).splitlines()
 
         # parse result - first 0:n/2 lines are 'checking ', the remaining ones
         # are the 'renaming' ones which we parse
-        for ln in xrange(int(len(mvlines) / 2), len(mvlines)):
+        for ln in range(int(len(mvlines) / 2), len(mvlines)):
             tokens = mvlines[ln].split(' to ')
             assert len(tokens) == 2, "Too many tokens in %s" % mvlines[ln]
 
@@ -893,19 +929,45 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
 
         return out
 
-    def commit(self, message, parent_commits=None, head=True, author=None, committer=None):
+    def commit(self, message, parent_commits=None, head=True, author=None,
+               committer=None, author_date=None, commit_date=None,
+               skip_hooks=False):
         """Commit the current default index file, creating a commit object.
         For more information on the arguments, see tree.commit.
 
         :note: If you have manually altered the .entries member of this instance,
                don't forget to write() your changes to disk beforehand.
+               Passing skip_hooks=True is the equivalent of using `-n`
+               or `--no-verify` on the command line.
         :return: Commit object representing the new commit"""
-        run_commit_hook('pre-commit', self)
+        if not skip_hooks:
+            run_commit_hook('pre-commit', self)
+
+            self._write_commit_editmsg(message)
+            run_commit_hook('commit-msg', self, self._commit_editmsg_filepath())
+            message = self._read_commit_editmsg()
+            self._remove_commit_editmsg()
         tree = self.write_tree()
         rval = Commit.create_from_tree(self.repo, tree, message, parent_commits,
-                                       head, author=author, committer=committer)
-        run_commit_hook('post-commit', self)
+                                       head, author=author, committer=committer,
+                                       author_date=author_date, commit_date=commit_date)
+        if not skip_hooks:
+            run_commit_hook('post-commit', self)
         return rval
+    
+    def _write_commit_editmsg(self, message):
+        with open(self._commit_editmsg_filepath(), "wb") as commit_editmsg_file:
+            commit_editmsg_file.write(message.encode(defenc))
+
+    def _remove_commit_editmsg(self):
+        os.remove(self._commit_editmsg_filepath())
+
+    def _read_commit_editmsg(self):
+        with open(self._commit_editmsg_filepath(), "rb") as commit_editmsg_file:
+            return commit_editmsg_file.read().decode(defenc)
+
+    def _commit_editmsg_filepath(self):
+        return osp.join(self.repo.common_dir, "COMMIT_EDITMSG")
 
     @classmethod
     def _flush_stdin_and_wait(cls, proc, ignore_stdout=False):
@@ -942,7 +1004,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             prior and after a file has been checked out
 
         :param kwargs:
-            Additional arguments to be pasesd to git-checkout-index
+            Additional arguments to be passed to git-checkout-index
 
         :return:
             iterable yielding paths to files which have been checked out and are
@@ -973,9 +1035,9 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             # line contents:
             stderr = stderr.decode(defenc)
             # git-checkout-index: this already exists
-            failed_files = list()
-            failed_reasons = list()
-            unknown_lines = list()
+            failed_files = []
+            failed_reasons = []
+            unknown_lines = []
             endings = (' already exists', ' is not in the cache', ' does not exist at stage', ' is unmerged')
             for line in stderr.splitlines():
                 if not line.startswith("git checkout-index: ") and not line.startswith("git-checkout-index: "):
@@ -1005,7 +1067,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
                 # END for each possible ending
             # END for each line
             if unknown_lines:
-                raise GitCommandError(("git-checkout-index", ), 128, stderr)
+                raise GitCommandError(("git-checkout-index",), 128, stderr)
             if failed_files:
                 valid_files = list(set(iter_checked_out_files) - set(failed_files))
                 raise CheckoutError(
@@ -1020,11 +1082,11 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             proc = self.repo.git.checkout_index(*args, **kwargs)
             proc.wait()
             fprogress(None, True, None)
-            rval_iter = (e.path for e in mviter(self.entries))
+            rval_iter = (e.path for e in self.entries.values())
             handle_stderr(proc, rval_iter)
             return rval_iter
         else:
-            if isinstance(paths, string_types):
+            if isinstance(paths, str):
                 paths = [paths]
 
             # make sure we have our entries loaded before we start checkout_index
@@ -1036,8 +1098,9 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             kwargs['as_process'] = True
             kwargs['istream'] = subprocess.PIPE
             proc = self.repo.git.checkout_index(args, **kwargs)
+            # FIXME: Reading from GIL!
             make_exc = lambda: GitCommandError(("git-checkout-index",) + tuple(args), 128, proc.stderr.read())
-            checked_out_files = list()
+            checked_out_files = []
 
             for path in paths:
                 co_path = to_native_path_linux(self._to_relative_path(path))
@@ -1047,11 +1110,11 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
                 try:
                     self.entries[(co_path, 0)]
                 except KeyError:
-                    dir = co_path
-                    if not dir.endswith('/'):
-                        dir += '/'
-                    for entry in mviter(self.entries):
-                        if entry.path.startswith(dir):
+                    folder = co_path
+                    if not folder.endswith('/'):
+                        folder += '/'
+                    for entry in self.entries.values():
+                        if entry.path.startswith(folder):
                             p = entry.path
                             self._write_path_to_stdin(proc, p, p, make_exc,
                                                       fprogress, read_from_stdout=False)
@@ -1160,7 +1223,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
         # index against anything but None is a reverse diff with the respective
         # item. Handle existing -R flags properly. Transform strings to the object
         # so that we can call diff on it
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             other = self.repo.rev_parse(other)
         # END object conversion
 
@@ -1169,7 +1232,7 @@ class IndexFile(LazyMixin, diff.Diffable, Serializable):
             cur_val = kwargs.get('R', False)
             kwargs['R'] = not cur_val
             return other.diff(self.Index, paths, create_patch, **kwargs)
-        # END diff against other item handlin
+        # END diff against other item handling
 
         # if other is not None here, something is wrong
         if other is not None:
